%% Read in four images (no, only three worked out)
image1 = imread('DSC_4732.jpg');
image2 = imread('DSC_4736.jpg');
%image3 = imread('DSC_4741.jpg');  %% This viewpoint does not see enough features
image4 = imread('DSC_4743.jpg');
%% Read in the feature points selected earlier with cpselect
load corresp.mat
%% Look at / edit the feature points
cpselect(image2, image1, points2, points1)
%%
cpselect(image4, image1, points4, points1)
%% Just show the feature points
figure; imshow(image1); hold on;
plot(points1(:,1), points1(:,2), 'yo');
figure; imshow(image2); hold on;
plot(points2(:,1), points2(:,2), 'yo');
figure; imshow(image4); hold on;
plot(points4(:,1), points4(:,2), 'yo');
%% Subtract off the means for better PCA performance
m = size(points1,1);
n = 3;
mean1 = mean(points1)
mean2 = mean(points2)
mean4 = mean(points4)
points1z = points1 - ones(m,1) * mean1;
points2z = points2 - ones(m,1) * mean2;
points4z = points4 - ones(m,1) * mean4;
%% Package up the W matrix
W = zeros(2*n,m);
W([1 4],:) = points1z';
W([2 5],:) = points2z';
W([3 6],:) = points4z';
%% Compute the SVD
[U,S,V] = svd(W);
figure; bar(diag(S));
%% Extract the points and cameras
cams = U(:,1:3) * S(1:3,1:3);
pts = V(:,1:3)';
%% Plot the points in 3D (note there is an arbitrary linear transform...)
figure; plot3(pts(1,:), pts(2,:), pts(3,:), '+'); axis equal
set(gca, 'PlotBoxAspectRatioMode', 'manual')
set(gca, 'CameraViewAngleMode', 'manual')
%% Plot their reprojections
projs1 = (cams([1 4],:) * pts)' + ones(m,1) * mean1;
projs2 = (cams([2 5],:) * pts)' + ones(m,1) * mean2;
projs4 = (cams([3 6],:) * pts)' + ones(m,1) * mean4;
figure; imshow(image1); hold on;
plot(points1(:,1), points1(:,2), 'yo');
plot(projs1(:,1), projs1(:,2), 'y+');
figure; imshow(image2); hold on;
plot(points2(:,1), points2(:,2), 'yo');
plot(projs2(:,1), projs2(:,2), 'y+');
figure; imshow(image4); hold on;
plot(points4(:,1), points4(:,2), 'yo');
plot(projs4(:,1), projs4(:,2), 'y+');
%% Call a more sophisticated implementation that squares up cameras
[C1, P1, C2, P2] = Tomasi(W);
cams = C1;
pts = P1;
%% Plot the points in 3D
figure; plot3(pts(1,:), pts(3,:), -pts(2,:), '+'); axis equal
set(gca, 'PlotBoxAspectRatioMode', 'manual')
set(gca, 'CameraViewAngleMode', 'manual')
